export type TData = unknown | object | any[];
export type VirtualTreeItem<TData> = TData & {
    id: number | string;
    isExpanded?: boolean;
    isLeaf?: boolean;
    parentId: number | string | null;
    children?: VirtualTreeItem<TData>[];
    icon: string;
    level: number;
};
export type VirtualTreeItemAccessorKey<T> = keyof VirtualTreeItem<T>;
export interface Rect {
    width: number;
    height: number;
}
export declare const observeElementRect: <T extends Element>(instance: TreeVirtualizer<T, any, any>, cb: (rect: Rect) => void) => (() => void) | undefined;
export interface VirtualizerOptions<TScrollElement extends Element | Window, TItemElement extends Element, TData> {
    data: VirtualTreeItem<TData>[];
    accessorKey: VirtualTreeItemAccessorKey<TData>;
    getScrollElement: () => TScrollElement | null;
    observeElementRect: (instance: TreeVirtualizer<TScrollElement, TItemElement, TData>, cb: (rect: Rect) => void) => void | (() => void);
    debug?: boolean;
    onChange?: (instance: TreeVirtualizer<TScrollElement, TItemElement, TData>, sync: boolean) => void;
    indexAttribute?: string;
    enabled?: boolean;
}
export declare class TreeVirtualizer<TScrollElement extends Element | Window, TItemElement extends Element, TData> {
    selectedId: string | number;
    visibleItems: VirtualTreeItem<TData>[];
    expandedNodes: Set<number | string>;
    options: Required<VirtualizerOptions<TScrollElement, TItemElement, TData>>;
    scrollElement: TScrollElement | null;
    targetWindow: (Window & typeof globalThis) | null;
    isScrolling: boolean;
    constructor(opts: VirtualizerOptions<TScrollElement, TItemElement, TData>);
    setOptions: (opts: VirtualizerOptions<TScrollElement, TItemElement, TData>) => void;
    private notify;
    private cleanup;
    _didMount: () => () => void;
    _willUpdate: () => void;
    private getIndexes;
    indexFromElement: (node: TItemElement) => number;
    getVirtualTreeItems: () => VirtualTreeItem<TData>[];
    /**
     * Sets the selected ID.
     *
     * @param id - The ID to be set as selected. Can be a string or a number.
     */
    setSelectedId(id: string | number): void;
    /**
     * Retrieves the currently selected ID.
     *
     * @returns {string | number} The ID of the selected item.
     */
    getSelectedId(): string | number;
    private updateVisibleNodes;
    /**
     * Adds or removes the node from the expandedNodes set
     * @param nodeId id of the node to toggle
     * @param index current index of the node
     * @returns
     */
    toggleNode(nodeId: number | string, index: number): void;
    private setVisibleNodes;
}
