import { TreeVirtualizer, observeElementRect } from './core';
import { derived, writable } from 'svelte/store';
function createTreeVirtualizerBase(initialOptions) {
    const virtualizer = new TreeVirtualizer(initialOptions);
    const originalSetOptions = virtualizer.setOptions;
    // eslint-disable-next-line prefer-const
    let virtualizerWritable;
    const setOptions = (options) => {
        const resolvedOptions = {
            ...virtualizer.options,
            ...options,
            onChange: options.onChange
        };
        originalSetOptions({
            ...resolvedOptions,
            onChange: (instance, sync) => {
                virtualizerWritable.set(instance);
                resolvedOptions.onChange?.(instance, sync);
            }
        });
        virtualizer._willUpdate();
    };
    virtualizerWritable = writable(virtualizer, () => {
        setOptions(initialOptions);
        return virtualizer._didMount();
    });
    return derived(virtualizerWritable, (instance) => Object.assign(instance, { setOptions }));
}
export function createTreeVirtualizer(options) {
    return createTreeVirtualizerBase({
        observeElementRect: observeElementRect,
        ...options
    });
}
